@@ -1,13 +1,16 @@
 import os
+import re
+import sys
 import warnings
-from typing import Any, Optional
+from typing import Any, Dict, List, Optional, Tuple, cast
+from urllib.parse import urlparse
 
-import cv2
+import cv2  # type: ignore
 import numpy as np
 import numpy.typing as npt
 import requests
 
-from .margin import HEIGHT_GAP, WIDTH_GAP
+from .gap import HEIGHT_GAP, WIDTH_GAP
 
 HEADERS = {
     "User-Agent": (
@@ -17,6 +20,24 @@ HEADERS = {
     )
 }
 
+VALID_HOSTS = (
+    "comic-action.com",
+    "comic-days.com",
+    "comic-gardo.com",
+    "comic-trail.com",
+    "comic-zenon.com",
+    "comicborder.com",
+    "comicbushi-web.com",
+    "feelweb.jp",
+    "kuragebunch.com",
+    "magcomi.com",
+    "pocket.shonenmagazine.com",
+    "shonenjumpplus.com",
+    "www.sunday-webry.com",
+    "tonarinoyj.jp",
+    "viewer.heros-web.com",
+)
+
 
 class NeedPurchase(Warning):
     pass
@@ -32,42 +53,60 @@ class GetJump:
         save_path: str = ".",
         overwrite: bool = True,
         only_first: bool = False,
-    ) -> tuple[Optional[str], str]:
+    ) -> Tuple[Optional[str], str, bool]:
+        self.__check_url(url)
         r = requests.get(url, headers=HEADERS)
-        if r.status_code != 200:
-            raise ConnectionError(r.status_code)
+        self.__check_content_type(r.headers["content-type"])
         j = r.json()["readableProduct"]
-        next = j["nextReadableProductUri"]
-        next = self.__check_next(next)
+        nxt = j["nextReadableProductUri"]
+        nxt = self.__check_next(nxt)
         series_title = j["series"]["title"].replace("/", "／")
         title = j["title"].replace("/", "／")
 
         save_dir = os.path.join(save_path, series_title, title)
         if os.path.exists(save_dir) and not overwrite:
-            return next, save_dir
+            print("already existed! (to overwrite, use `-o`)", file=sys.stderr)
+            return nxt, save_dir, False
         os.makedirs(save_dir, exist_ok=True)
 
         if not j["isPublic"] and not j["hasPurchased"]:
             warnings.warn(title, NeedPurchase, stacklevel=1)
-            return next, save_dir
+            return nxt, save_dir, False
         else:
-            pages = j["pageStructure"]["pages"]
+            pages = [p for p in j["pageStructure"]["pages"] if "src" in p]
 
         self.__save_images(pages, save_dir, only_first)
 
-        return next, save_dir
+        return nxt, save_dir, True
 
     @staticmethod
-    def __check_next(next: Optional[str]) -> Optional[str]:
-        return next + ".json" if type(next) is str else next
+    def is_valid_uri(url: str) -> bool:
+        o = urlparse(url)
+        return (
+            type(url) is str
+            and o.scheme == "https"
+            and o.hostname in VALID_HOSTS
+            and bool(re.match(r"^/episode/[0-9]+\.json$", o.path))
+        )
+
+    def __check_url(self, url: str) -> None:
+        if not self.is_valid_uri(url):
+            raise ValueError(f"'{url}' is not valid url.")
+
+    @staticmethod
+    def __check_content_type(type_: str) -> None:
+        if "application/json" not in type_:
+            raise TypeError(type_ + " is not application/json")
+
+    @staticmethod
+    def __check_next(nxt: Optional[str]) -> Optional[str]:
+        return nxt + ".json" if type(nxt) is str else nxt
 
     def __save_images(
-        self, pages: list[Any], save_dir: str, only_first: bool = False
+        self, pages: List[Any], save_dir: str, only_first: bool = False
     ) -> None:
         imgs = []
         for page in pages:
-            if "src" not in page:
-                continue
             img = self.__get_image(page)
             imgs.append(img)
             if only_first:
@@ -80,7 +119,7 @@ class GetJump:
             )
             cv2.imwrite(save_img_path, img)
 
-    def __get_image(self, image_dic: dict[str, Any]) -> npt.ArrayLike:
+    def __get_image(self, image_dic: Dict[str, Any]) -> npt.ArrayLike:
         src = image_dic["src"]
         img = requests.get(src, stream=True).raw
         img = np.asarray(bytearray(img.read()), dtype="uint8")
@@ -99,7 +138,7 @@ class GetJump:
                 pieces[y][x] = piece
 
         img = cv2.vconcat([cv2.hconcat(x) for x in pieces])
-        return img
+        return cast(npt.ArrayLike, img)
 
     @staticmethod
     def __get_height_gap(height: int) -> int:
@@ -110,7 +149,7 @@ class GetJump:
                     % height
                 )
             else:
-                return HEIGHT_GAP[height]  # type: ignore
+                return HEIGHT_GAP[height]
         else:
             raise ValueError(
                 "Unfamiliar height (please let me know with issue <https://git.io/J2jV3>): %d"
@@ -126,7 +165,7 @@ class GetJump:
                     % width
                 )
             else:
-                return WIDTH_GAP[width]  # type: ignore
+                return WIDTH_GAP[width]
         else:
             raise ValueError(
                 "Unfamiliar width (please let me know with issue <https://git.io/J2jV3>): %d"
